// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "BufferedStream.h"

#include "ReadingBuffer.h"
#include "WritingBuffer.h"

#include <unistd.h>
#include <poll.h>

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::BufferedStream;

//------------------------------------------------------------------------------

BufferedStream::BufferedStream(Waiter* waiter, int fd,
                                   size_t readingCapacity,
                                   size_t writingCapacity,
                                   short events) :
    Waitable(waiter, fd, events),
    readingBuffer( (readingCapacity==0) ? 0 :
                   new ReadingBuffer(readingCapacity, this)),
    writingBuffer( (writingCapacity==0) ? 0 :
                   new WritingBuffer(writingCapacity, this))
{
}

//------------------------------------------------------------------------------

BufferedStream::~BufferedStream()
{
    delete writingBuffer;
    delete readingBuffer;
}

//------------------------------------------------------------------------------

ssize_t BufferedStream::read(void* dest, size_t length)
{
    ssize_t result = ::read(fd, dest, length);
    if (result<0 && errno!=EAGAIN && errno!=EWOULDBLOCK) {
        setErrorCodeFromErrno();
    }
    return result;
}

//------------------------------------------------------------------------------

ssize_t BufferedStream::write(const void* src, size_t length)
{
    ssize_t result = ::write(fd, src, length);
    if (result<0 && errno!=EAGAIN && errno!=EWOULDBLOCK) {
        setErrorCodeFromErrno();
    }
    return result;
}

//------------------------------------------------------------------------------

bool BufferedStream::isReady()
{
    return false;
}

//------------------------------------------------------------------------------

void BufferedStream::handleEvents(short ev)
{
    if ((ev&POLLIN)==POLLIN && readingBuffer!=0) {
        readingBuffer->read();
    }
    if ((ev&POLLOUT)==POLLOUT && writingBuffer!=0) {
        writingBuffer->write();
    }
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
